"use strict";Object.defineProperty(exports, "__esModule", {value: true});var _biggystring = require('biggystring');








var _cleaners = require('cleaners');










var _types = require('edge-core-js/types');









var _swaphelpers = require('../swap-helpers');
var _utils = require('../util/utils');

const pluginId = 'changenow'

const swapInfo = {
  pluginId,
  isDex: false,
  displayName: 'Change NOW',
  supportEmail: 'support@changenow.io'
}

const asInitOptions = _cleaners.asObject.call(void 0, {
  apiKey: _cleaners.asString
})

const orderUri = 'https://changenow.io/exchange/txs/'
const uri = 'https://api.changenow.io/v2/'

const INVALID_CURRENCY_CODES = {
  from: {
    optimism: ['VELO']
  },
  to: {
    zcash: ['ZEC']
  }
}

// Network names that don't match parent network currency code
// See https://changenow.io/currencies for list of supported currencies
const MAINNET_CODE_TRANSCRIPTION = {
  binancesmartchain: 'BSC'
}

 function makeChangeNowPlugin(
  opts
) {
  const { io } = opts
  const { fetchCors = io.fetch } = io
  const { apiKey } = asInitOptions(opts.initOptions)

  const headers = {
    'Content-Type': 'application/json',
    'x-changenow-api-key': apiKey
  }

  const fetchSwapQuoteInner = async (
    request,
    opts
  ) => {
    const { promoCode } = opts
    const { nativeAmount } = request

    // Grab addresses:
    const [fromAddress, toAddress] = await Promise.all([
      _utils.getAddress.call(void 0, request.fromWallet),
      _utils.getAddress.call(void 0, request.toWallet)
    ])

    const {
      fromCurrencyCode,
      toCurrencyCode,
      fromMainnetCode,
      toMainnetCode
    } = _swaphelpers.getCodesWithTranscription.call(void 0, request, MAINNET_CODE_TRANSCRIPTION)
    const currencyString = `fromCurrency=${fromCurrencyCode}&toCurrency=${toCurrencyCode}&fromNetwork=${fromMainnetCode}&toNetwork=${toMainnetCode}`

    async function createOrder(
      flow,
      isSelling,
      largeDenomAmount
    ) {
      const type = isSelling ? 'direct' : 'reverse'

      // Get rateId and Date
      const url = `exchange/estimated-amount?flow=${flow}&useRateId=${String(
        flow === 'fixed-rate'
      )}&${
        isSelling ? 'fromAmount' : 'toAmount'
      }=${largeDenomAmount}&type=${type}&${currencyString}`
      const exchangeAmountResponse = await fetchCors(uri + url, { headers })
      const exchangeAmountResponseJson = await exchangeAmountResponse.json()

      if (exchangeAmountResponseJson.error != null)
        throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)

      const { rateId, validUntil } = asExchange(exchangeAmountResponseJson)

      // Create order
      const orderBody = {
        fromCurrency: fromCurrencyCode,
        toCurrency: toCurrencyCode,
        fromNetwork: fromMainnetCode,
        toNetwork: toMainnetCode,
        fromAmount: isSelling ? largeDenomAmount : '',
        toAmount: isSelling ? '' : largeDenomAmount,
        type,
        address: toAddress,
        refundAddress: fromAddress,
        flow,
        rateId,
        payload: { promoCode }
      }

      const orderResponse = await fetchCors(uri + 'exchange', {
        method: 'POST',
        body: JSON.stringify(orderBody),
        headers
      })
      if (!orderResponse.ok) {
        const text = await orderResponse.text()
        throw new Error(
          `ChangeNow call returned error code ${orderResponse.status}, ${text}`
        )
      }
      const orderResponseJson = await orderResponse.json()

      return { ...asOrder(orderResponseJson), validUntil }
    }

    async function swapSell(
      flow
    ) {
      const largeDenomAmount = await request.fromWallet.nativeToDenomination(
        nativeAmount,
        fromCurrencyCode
      )

      // Get min and max
      const marketRangeResponse = await fetchCors(
        uri + `exchange/range?flow=${flow}&${currencyString}`,
        { headers }
      )
      const marketRangeResponseJson = await marketRangeResponse.json()

      if (marketRangeResponseJson.error != null)
        throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)

      const { minAmount, maxAmount } = asMarketRange(marketRangeResponseJson)

      if (_biggystring.lt.call(void 0, largeDenomAmount, minAmount.toString())) {
        const minNativeAmount = await request.fromWallet.denominationToNative(
          minAmount.toString(),
          fromCurrencyCode
        )
        throw new (0, _types.SwapBelowLimitError)(swapInfo, minNativeAmount)
      }

      if (maxAmount != null && _biggystring.gt.call(void 0, largeDenomAmount, maxAmount.toString())) {
        const maxNativeAmount = await request.fromWallet.denominationToNative(
          maxAmount.toString(),
          fromCurrencyCode
        )
        throw new (0, _types.SwapAboveLimitError)(swapInfo, maxNativeAmount)
      }

      const {
        toAmount,
        payinAddress,
        payinExtraId,
        id,
        validUntil
      } = await createOrder(flow, true, largeDenomAmount)

      const toNativeAmount = await request.toWallet.denominationToNative(
        toAmount.toString(),
        toCurrencyCode
      )

      const spendInfo = {
        currencyCode: fromCurrencyCode,
        spendTargets: [
          {
            nativeAmount,
            publicAddress: payinAddress,
            uniqueIdentifier: payinExtraId
          }
        ],
        networkFeeOption: fromCurrencyCode === 'BTC' ? 'high' : 'standard',
        swapData: {
          orderId: id,
          orderUri: orderUri + id,
          isEstimate: flow === 'standard',
          payoutAddress: toAddress,
          payoutCurrencyCode: toCurrencyCode,
          payoutNativeAmount: toNativeAmount,
          payoutWalletId: request.toWallet.id,
          plugin: { ...swapInfo },
          refundAddress: fromAddress
        }
      }

      return {
        request,
        spendInfo,
        swapInfo,
        fromNativeAmount: nativeAmount,
        expirationDate:
          validUntil != null
            ? _swaphelpers.ensureInFuture.call(void 0, validUntil)
            : new Date(Date.now() + 1000 * 60)
      }
    }

    async function swapBuy(flow) {
      // Skip min/max check when requesting a purchase amount
      const largeDenomAmount = await request.toWallet.nativeToDenomination(
        nativeAmount,
        toCurrencyCode
      )

      const {
        fromAmount,
        payinAddress,
        payinExtraId,
        id,
        validUntil
      } = await createOrder(flow, false, largeDenomAmount)

      const fromNativeAmount = await request.fromWallet.denominationToNative(
        fromAmount.toString(),
        fromCurrencyCode
      )

      const spendInfo = {
        currencyCode: fromCurrencyCode,
        spendTargets: [
          {
            nativeAmount: fromNativeAmount,
            publicAddress: payinAddress,
            uniqueIdentifier: payinExtraId
          }
        ],
        networkFeeOption: fromCurrencyCode === 'BTC' ? 'high' : 'standard',
        swapData: {
          orderId: id,
          orderUri: orderUri + id,
          isEstimate: false,
          payoutAddress: toAddress,
          payoutCurrencyCode: toCurrencyCode,
          payoutNativeAmount: nativeAmount,
          payoutWalletId: request.toWallet.id,
          plugin: { ...swapInfo },
          refundAddress: fromAddress
        }
      }

      return {
        request,
        spendInfo,
        swapInfo,
        fromNativeAmount,
        expirationDate:
          validUntil != null
            ? _swaphelpers.ensureInFuture.call(void 0, validUntil)
            : new Date(Date.now() + 1000 * 60)
      }
    }

    const { quoteFor } = request

    // Try them all
    if (quoteFor === 'from') {
      try {
        return await swapSell('fixed-rate')
      } catch (e) {
        try {
          return await swapSell('standard')
        } catch (e2) {
          // Should throw the fixed-rate error
          throw e
        }
      }
    } else {
      return await swapBuy('fixed-rate')
    }
  }

  const out = {
    swapInfo,

    async fetchSwapQuote(
      req,
      userSettings,
      opts
    ) {
      const request = _utils.convertRequest.call(void 0, req)

      _swaphelpers.checkInvalidCodes.call(void 0, INVALID_CURRENCY_CODES, request, swapInfo)

      const newRequest = await _swaphelpers.getMaxSwappable.call(void 0, 
        fetchSwapQuoteInner,
        request,
        opts
      )
      const swapOrder = await fetchSwapQuoteInner(newRequest, opts)
      return await _swaphelpers.makeSwapPluginQuote.call(void 0, swapOrder)
    }
  }
  return out
} exports.makeChangeNowPlugin = makeChangeNowPlugin;

/**
 * An optional value, where a blank string means undefined.
 */
 function asOptionalBlank(
  cleaner
) {
  return function asIgnoredBlank(raw) {
    if (raw == null || raw === '') return
    return cleaner(raw)
  }
} exports.asOptionalBlank = asOptionalBlank;

const asMarketRange = _cleaners.asObject.call(void 0, {
  maxAmount: _cleaners.asMaybe.call(void 0, _cleaners.asNumber),
  minAmount: _cleaners.asNumber
})

const asExchange = _cleaners.asObject.call(void 0, {
  rateId: _cleaners.asOptional.call(void 0, _cleaners.asString),
  validUntil: _cleaners.asOptional.call(void 0, _cleaners.asDate)
})

const asOrder = _cleaners.asObject.call(void 0, {
  fromAmount: _cleaners.asNumber,
  toAmount: _cleaners.asNumber,
  payinAddress: _cleaners.asString,
  payinExtraId: asOptionalBlank(_cleaners.asString),
  id: _cleaners.asString
})


